#!/usr/bin/env python 
import rospy
from radar_sensor_plugin.msg import radar_data
import numpy as np
import matplotlib.mlab as mlab
import matplotlib.pyplot as plt
import matplotlib.animation as Animation
from matplotlib import style
from numpy import convolve
import matplotlib

style.use('fivethirtyeight')

INTERVAL = 5 # seconds to sleep between data update calls

#global variables to hold the data
xdata1 = [0]*10 
ydata1 = [0]*10
xdata2 = [0]*10
ydata2 = [0]*10
xdata3 = [0]*10
ydata3 = [0]*10
xdata4 = [0]*10
ydata4 = [0]*10


def callback1(data):
    global xdata1, ydata1
    xdata1 = data.range
    ydata1 = data.histogram_bar

def callback2(data):
    global xdata2, ydata2
    xdata2 = data.range
    ydata2 = data.histogram_bar
    
def callback3(data):
    global xdata3, ydata3
    xdata3 = data.range
    ydata3 = data.histogram_bar

def callback4(data):
    global xdata4, ydata4
    xdata4 = data.range
    ydata4 = data.histogram_bar

def animate(i):
    global ydata1, xdata1, ydata2, xdata2, ydata3, xdata3, ydata4, xdata4, line1, line2, line3, line4

    # update the data
    line1.set_data(xdata1, ydata1)
    line2.set_data(xdata2, ydata2)
    line3.set_data(xdata3, ydata3)
    line4.set_data(xdata4, ydata4)

    # update the scales to fit:
    if ydata1:
        ax1.set_ylim(-0.02, max(ydata1))
    if xdata1:
        ax1.set_xlim(min(xdata1), max(xdata1))

    if ydata2:
        ax2.set_ylim(-0.02, 0.7)
    if xdata2:
        ax2.set_xlim(min(xdata2), max(xdata2))
 
    if ydata3:
        ax3.set_ylim(-0.02, 0.7)
    if xdata3:
        ax3.set_xlim(min(xdata3), max(xdata3))
 
    if ydata4:
        ax4.set_ylim(-0.02, 0.7)
    if xdata4:
        ax4.set_xlim(min(xdata4), max(xdata4))

    return line1, line2, line3, line4



def listener():

    # In ROS, nodes are uniquely named. If two nodes with the same
    # node are launched, the previous one is kicked off. The
    # anonymous=True flag means that rospy will choose a unique
    # name for our 'listener' node so that multiple listeners can
    # run simultaneously.

    rospy.init_node('plotter', anonymous=True)

    rospy.Subscriber("radar1", radar_data, callback1)
    rospy.Subscriber("radar2", radar_data, callback2)
    rospy.Subscriber("radar3", radar_data, callback3)
    rospy.Subscriber("radar4", radar_data, callback4)
    
    global ax1, ax2, ax3, ax4, line1, line2, line3, line4
    fig = plt.figure()
    ax1 = fig.add_subplot(221)
    ax2 = fig.add_subplot(222)
    ax3 = fig.add_subplot(223)
    ax4 = fig.add_subplot(224)
    ax1.set_title('Sensor 1, front radar')
    ax2.set_title('Sensor 2, right radar')
    ax3.set_title('Sensor 3, back radar')
    ax4.set_title('Sensor 4, left radar')
    ax1.set_xlabel('Range')
    ax1.set_ylabel('Amplitude')
    ax2.set_xlabel('Range')
    ax2.set_ylabel('Amplitude')
    ax3.set_xlabel('Range')
    ax3.set_ylabel('Amplitude')
    ax4.set_xlabel('Range')
    ax4.set_ylabel('Amplitude')
    line1, = ax1.plot([], [], lw=2) # initial empty graph
    line2, = ax2.plot([], [], lw=2) # initial empty graph
    line3, = ax3.plot([], [], lw=2) # initial empty graph
    line4, = ax4.plot([], [], lw=2) # initial empty graph
    ani = Animation.FuncAnimation(fig, animate, interval=200)
    plt.show() # start animation loop


    # spin() simply keeps python from exiting until this node is stopped
    rospy.spin()
    
if __name__ == '__main__':
    listener()
