#!/usr/bin/env python 
import rospy
from radar_sensor_plugin.msg import radar_data
import numpy as np
import matplotlib.mlab as mlab
import matplotlib.pyplot as plt
import matplotlib.animation as Animation
import matplotlib.cm as cm
from matplotlib import style
from scipy.interpolate import interp1d
import matplotlib
from math import cos, sin

style.use('fivethirtyeight')

# Global variables to hold the data
range_data = []
amplitude_data = []

quadrotor_position = 0
interp_x_pos = interp1d([-19.0, 16.0], [0, 250])
interp_y_pos = interp1d([-20.5, 14.5], [0, 250])
image = np.zeros((250, 250))
image[0][0] = 0.7

def callback(data):
    global range_data, amplitude_data, quadrotor_position
    range_data = data.range
    amplitude_data = data.histogram_bar
    quadrotor_position = data.position

def animate(i):
    im.set_array(update_grid())
    return im,
    
def update_grid():
    global interp_x_pos, interp_y_pos, image, quadrotor_position, amplitude_data, range_data

    if amplitude_data:
        x_pos = quadrotor_position[0]
        y_pos = quadrotor_position[1]
        yaw   = quadrotor_position[3]

        for i in range(len(range_data)):
            x_grid_pos = round(interp_x_pos(x_pos + cos(yaw)*range_data[i]), 0)
            y_grid_pos = round(interp_y_pos(y_pos + sin(yaw)*range_data[i]), 0)
            image[y_grid_pos][x_grid_pos] = amplitude_data[i]
      
    return image
    
def listener():

    # In ROS, nodes are uniquely named. If two nodes with the same
    # node are launched, the previous one is kicked off. The
    # anonymous=True flag means that rospy will choose a unique
    # name for our 'listener' node so that multiple listeners can
    # run simultaneously.

    rospy.init_node('plotter', anonymous=True)

    rospy.Subscriber("radar1", radar_data, callback)
    
    global im
    fig = plt.figure()

    im = plt.imshow(update_grid(), animated=True, extent=[-5.0, 2.0, -6.0, 1.0])
#, cmap="hot"
    ani = Animation.FuncAnimation(fig, animate, interval=200)
    plt.show() # start animation loop

    # spin() simply keeps python from exiting until this node is stopped
    rospy.spin()
    
if __name__ == '__main__':
    listener()
