#ifndef PLUGINS_GAZEBO_ROS_RADAR_2_H 
#define PLUGINS_GAZEBO_ROS_RADAR_2_H

#include <gazebo/common/Plugin.hh>

#include <ros/ros.h>
#include <ros/callback_queue.h>

#include <sensor_msgs/Range.h>
#include <radar_sensor_plugin/radar_data.h>

#include <array>

namespace gazebo
{

class GazeboRosRadar2 : public SensorPlugin
{
public:
  GazeboRosRadar2();
  virtual ~GazeboRosRadar2();

protected:
  /// \brief Load the sensor plugin.
  /// \param[in] _sensor Pointer to the sensor that loaded this plugin.
  /// \param[in] _sdf SDF element that describes the plugin.
  virtual void Load(sensors::SensorPtr _sensor, sdf::ElementPtr _sdf);

  virtual void Reset();
  virtual void Update();

private:
  /// \brief The parent World
  physics::WorldPtr world_;

  std::string sensor_carrier_name;

  sensors::RaySensorPtr sensor_;

  ros::NodeHandle* node_handle_;
  ros::Publisher publisher_[4];

  sensor_msgs::Range range_;
  radar_sensor_plugin::radar_data msg_;

  std::string namespace_;
  std::string topic_[4];
  std::string frame_id_;

  event::ConnectionPtr updateConnection;

  int ray_total_count_; 
  int initialize_sensor_iteration_;
  void PublishRadarData(std::vector<std::vector<double>> array_ray_info_m, ros::Publisher & pub);

  // Private functions and variables for handling ray tracing and angle calculations
  void FillRayMap(std::map<int, std::array<double, 3> >& ray_angle_map, double azimuth_opening, double ray_count);
  double CalculateVectorAngle(double r1, double r2, double r3, double u1, double u2, double u3);
  void RotateRayMap(std::map<int, std::array<double, 3> >& ray_angle_map, double yaw, double ray_count);
  void RotateVector(std::array<double, 3>& vector, double yaw);
};

} // namespace gazebo

#endif //GAZEBO_ROS_Radar_H

